{ nixpkgs ? import <nixpkgs> {}, compiler ? "ghc843", doBenchmark ? false }:

let

  inherit (nixpkgs) pkgs;

  sdl2Nek0 = with haskellPackages; callPackage (
    { mkDerivation, base, bytestring, exceptions, linear, SDL2
    , StateVar, stdenv, text, transformers, vector
    }:
    mkDerivation {
      pname = "sdl2";
      version = "2.4.0.1";
      sha256 = "9a898d0acd6d7a4f341b3127d2c44e64b577e1128bc0b4d93a17ac510cb5644d";
      isLibrary = true;
      isExecutable = true;
      enableSeparateDataOutput = true;
      libraryHaskellDepends = [
        base bytestring exceptions linear StateVar text transformers vector
      ];
      doCheck = false;
      librarySystemDepends = [ SDL2 ];
      libraryPkgconfigDepends = [ SDL2 ];
      description = "Both high- and low-level bindings to the SDL library (version 2.0.4+).";
      license = stdenv.lib.licenses.bsd3;
    }) {};

  nanovgNeko = with haskellPackages; callPackage ({
    mkDerivation, base, bytestring, c2hs, containers, glew, hspec,
    inline-c, libGL, libGLU, QuickCheck, text, vector, pkgconfig }:
    mkDerivation {
      pname = "nanovg";
      version = "0.6.0.0";
      sha256 = "e35a2772f2a2e71916013cab70eeb8eedd6f66b5490ddf7ce237c8c76895d496";
      editedCabalFile = "1lhfsq50r9bdby7mfscw8ih3wsz2m19lr2a2z9b6diaf5kz76vij";
      revision = "1";
      isLibrary = true;
      isExecutable = true;
      doCheck = false;
      libraryHaskellDepends = [ base bytestring containers text vector ];
      librarySystemDepends = [ glew libGL libGLU pkgconfig ];
      libraryToolDepends = [ c2hs ];
      testHaskellDepends = [ base containers hspec inline-c QuickCheck ];
      homepage = "https://github.com/cocreature/nanovg-hs";
      description = "Haskell bindings for nanovg";
      license = pkgs.stdenv.lib.licenses.isc;
    }) {};

  agNek0 = with haskellPackages; callPackage (
    { mkDerivation, array, base, base-orphans, containers
    , criterion, deepseq, extra, QuickCheck, stdenv
    }:
    mkDerivation {
      pname = "algebraic-graphs";
      version = "0.1.1.1";
      sha256 = "8747a7d6f7a328b6073c1904883087a6a8e9a02d3bd8a5e35174c5f1c1cd1231";
      libraryHaskellDepends = [
        array base base-compat containers deepseq
      ];
      testHaskellDepends = [
        base bcNek0 base-orphans containers extra QuickCheck
      ];
      benchmarkHaskellDepends = [
        base base-compat containers criterion
      ];
      homepage = "https://github.com/snowleopard/alga";
      doCheck = false;
      description = "A library for algebraic graph construction and transformation";
      license = stdenv.lib.licenses.mit;
    }) {};

  bcNek0 = with haskellPackages; callpackage (
    { mkDerivation, base, stdenv, unix }:
    mkDerivation {
      pname = "base-compat";
      version = "0.9.3";
      libraryHaskellDepends = [ base unix ];
      description = "A compatibility layer for base";
      doCheck = false;
      license = stdenv.lib.licenses.mit;
    }) {};

  affectionNeko = with haskellPackages; callPackage ({ mkDerivation
    , base, bytestring, clock, containers, glib, linear, monad-loops, monad-parallel
    , mtl, OpenGL, stdenv, stm, text, uuid, vector }:
    mkDerivation {
      pname = "affection";
      version = "0.0.0.9";
      #src = pkgs.fetchFromGitHub {
      #  owner = "nek0";
      #  repo = "affection";
      #  rev = "33c99b8888328e4ed17f5c65ac49f5eab2645549";
      #  sha256 = "0psqxqj1a8l5fia49ay2pb72kjnw5i54m6dcmrpz5hi1654aznll";
      #};
      src = ../affection;
      configureFlags = [ "-fdebug" ];
      revision = "1";
      isLibrary = true;
      libraryHaskellDepends = [
        base bytestring clock containers glib linear monad-loops
        monad-parallel mtl OpenGL sdl2Nek0 stm text uuid vector
      ];
      librarySystemDepends = [ ];
      libraryToolDepends = [  ];
      testHaskellDepends = [ ];
      homepage = "https://github.com/nek0/affection";
      description = "A simple Game Engine in Haskell using SDL";
      license = pkgs.stdenv.lib.licenses.gpl3;
      hydraPlatforms = pkgs.stdenv.lib.platforms.none;
    }) { };

  ecstasyNeko = with haskellPackages; callPackage ({
    mkDerivation, base, containers, kan-extensions, mtl, stdenv
    , transformers
    }:
    mkDerivation {
      pname = "ecstasy";
      version = "0.2.1.0";
      sha256 = "915942d3b8c3d61b98e5b2e825387d48cf3c2d17acdb2d377cb516c26c0fcbc3";
      libraryHaskellDepends = [
        base containers kan-extensions mtl transformers
      ];
      homepage = "http://github.com/isovector/ecstasy/";
      description = "A GHC.Generics based entity component system.";
      license = stdenv.lib.licenses.bsd3;
    }) {};

  f = { mkDerivation, astar, base, containers, linear
      , matrix, OpenGL, random, stdenv, stm, text, unordered-containers
      , vector, JuicyPixels, JuicyPixels-extra, bytestring, mtl
      }:
      mkDerivation {
        pname = "tracer-game";
        version = "0.0.0.0";
        src = ./.;
        isLibrary = false;
        isExecutable = true;
        enableExecutableProfiling = true;
        executableHaskellDepends = [
          affectionNeko astar base containers ecstasyNeko linear matrix nanovgNeko
          OpenGL random sdl2Nek0 stm text unordered-containers vector JuicyPixels
          JuicyPixels-extra bytestring agNek0 mtl
        ];
        license = stdenv.lib.licenses.gpl3;
      };

  haskellPackages = if compiler == "default"
                       then pkgs.haskellPackages
                       else pkgs.haskell.packages.${compiler};

  variant = if doBenchmark then pkgs.haskell.lib.doBenchmark else pkgs.lib.id;

  drv = variant (haskellPackages.callPackage f {});

in

  if pkgs.lib.inNixShell then drv.env else drv
